@@ -5,6 +5,39 @@ using namespace simdjson;
 
 namespace misc_tests {
   using namespace std;
+bool issue2322() {
+  TEST_START();
+  std::vector<std::pair<std::string, bool>> examples = {{R"("hello")", false},
+                                         {R"(\"hello)", true},
+                                         {R"("hello\")", true},
+                                         {R"("hel\"lo")", false},
+                                         {R"("hel\\lo")", false},
+                                         {R"(\"hel\\\"lo\")", true},
+                                         {R"(\\"hel\\\"lo\")", false}};
+  for (std::pair<std::string, bool> v : examples) {
+    ASSERT_EQUAL(ondemand::raw_json_string::is_free_from_unescaped_quote(v.first),
+                 v.second);
+    ASSERT_EQUAL(ondemand::raw_json_string::is_free_from_unescaped_quote(v.first.c_str()),
+                 v.second);
+  }
+  TEST_SUCCEED();
+}
+#if SIMDJSON_EXCEPTIONS
+  // user reported an asan error:
+  bool issue2199() {
+    TEST_START();
+    static constexpr std::string_view kJsonString = R"( { "name": "name", "version": 100, } )";
+    try {
+      simdjson::padded_string buffer{kJsonString};
+      simdjson::ondemand::parser parser;
+      simdjson::ondemand::document document = parser.iterate(buffer);
+      (void)document;
+    } catch (simdjson::simdjson_error& /*error*/) {
+      std::cerr << "Caught simdjson_error" << std::endl;
+    }
+    TEST_SUCCEED();
+  }
+#endif
   bool issue1981_success() {
     auto error_phrase = R"(false)"_padded;
     TEST_START();
@@ -521,6 +554,24 @@ namespace misc_tests {
     TEST_SUCCEED();
   }
 
+  simdjson_warn_unused bool issue2312() {
+    TEST_START();
+    std::string init_string = R"("abc":)";
+    init_string.resize(init_string.size() + simdjson::SIMDJSON_PADDING);
+    simdjson::padded_string_view padded_view{init_string.data(), 5, init_string.size()};
+    simdjson::ondemand::parser parser;
+    simdjson::ondemand::document doc;
+    ASSERT_SUCCESS(parser.iterate(padded_view).get(doc));
+    std::string_view abc;
+    ASSERT_SUCCESS(doc.get_string().get(abc));
+    ASSERT_EQUAL(abc, "abc");
+    ASSERT_SUCCESS(parser.iterate(padded_view).get(doc));
+    std::string_view raw;
+    ASSERT_SUCCESS(doc.raw_json().get(raw));
+    ASSERT_EQUAL(raw, "\"abc\"");
+    TEST_SUCCEED();
+  }
+
   simdjson_warn_unused bool big_integer() {
     TEST_START();
     simdjson::ondemand::parser parser;
@@ -532,7 +583,7 @@ namespace misc_tests {
     string_view token;
     ASSERT_SUCCESS(o["value"].raw_json_token().get(token));
     ASSERT_EQUAL(token, "12321323213213213213213213213211223");
-    return true;
+    TEST_SUCCEED();
   }
   simdjson_warn_unused bool big_integer_in_string() {
     TEST_START();
@@ -545,12 +596,12 @@ namespace misc_tests {
     string_view token;
     ASSERT_SUCCESS(o["value"].raw_json_token().get(token));
     ASSERT_EQUAL(token, "\"12321323213213213213213213213211223\"");
-    return true;
+    TEST_SUCCEED();
   }
   simdjson_warn_unused bool test_raw_json_token(string_view json, string_view expected_token, int expected_start_index = 0) {
     string title("'");
     title.append(json.data(), json.length());
-    title += std::string("'");
+    title += "'"s;
     padded_string json_padded = json;
     SUBTEST(title, test_ondemand_doc(json_padded, [&](auto doc) {
       string_view token;
@@ -558,17 +609,17 @@ namespace misc_tests {
       ASSERT_EQUAL( token, expected_token );
       // Validate the text is inside the original buffer
       ASSERT_EQUAL( reinterpret_cast<const void*>(token.data()), reinterpret_cast<const void*>(&json_padded.data()[expected_start_index]));
-      return true;
+      TEST_SUCCEED();
     }));
 
     // Test values
     auto json_in_hash = string(R"({"a":)");
     json_in_hash.append(json.data(), json.length());
-    json_in_hash += std::string("}");
+    json_in_hash += "}"s;
     json_padded = json_in_hash;
-    title = std::string("'");
+    title = "'"s;
     title.append(json_in_hash.data(), json_in_hash.length());
-    title += std::string("'");
+    title += "'"s;
     SUBTEST(title, test_ondemand_doc(json_padded, [&](auto doc) {
       string_view token;
       ASSERT_SUCCESS( doc["a"].raw_json_token().get(token) );
@@ -576,10 +627,9 @@ namespace misc_tests {
       // Validate the text is inside the original buffer
       // Adjust for the {"a":
       ASSERT_EQUAL( reinterpret_cast<const void*>(token.data()), reinterpret_cast<const void*>(&json_padded.data()[5+expected_start_index]));
-      return true;
+      TEST_SUCCEED();
     }));
-
-    return true;
+    TEST_SUCCEED();
   }
 
   bool raw_json_token() {
@@ -605,6 +655,11 @@ namespace misc_tests {
 
   bool run() {
     return
+           issue2322() &&
+           issue2312() &&
+#if SIMDJSON_EXCEPTIONS
+           issue2199() &&
+#endif
            skipbom() &&
            issue1981_success() &&
            issue1981_failure() &&
